/* davespaceGMapFromXML - A program to turn an XML file into a Google map which filters 
 *                        by type.
 * Author:        William David Mayo
 * Contact Email: pobocks@gmail.com
 * Version: 0.8
 *
 * This program parses an XML file using the browser's parser,
 * then processes that information and displays a google map
 * with markers reflecting the contents of the XML file.
 *
 * This program adds the entities "map" and "davespaceGMapFromXML"
 * to the global namespace.
 *
 * Additional notes on data structures:
 *    davespaceGMapFromXML.locations is an array of objects containing all relevant data from
 *       locations in the XML file.
 *    davespaceGMapFromXML.markers is a similar array filled with the markers referring to those
 *       locations.  They are linked to each other by common numerical subscripting. 
 *    It is VASTLY important that after initialization, nothing is ever deleted from either markers
 *       or locations, unless it is also deleted in an identical manner from the other.  
 *       In practical terms, there is no reason to ever delete anything from either. So don't.
 */

// Establishes a namespace for the program.
var davespaceGMapFromXML = {};

// "Global" properties (Within the namespace).
davespaceGMapFromXML.locations = new Array();
davespaceGMapFromXML.markers = new Array();

/* Parses the XML and puts it into the "global" locations object. */
davespaceGMapFromXML.parseXML = function ()
{			      
    try 
    {
	var request = GXmlHttp.create();
	request.open("GET", "locations.xml", false);
	request.send(null);
	davespaceGMapFromXML.xmlDoc = request.responseXML;

	// Let's alias the ridiculously long global property names, shall we?
	var locations = davespaceGMapFromXML.locations;
	var xmlDoc = davespaceGMapFromXML.xmlDoc;
	
	/* First we take all the locations from the XML file and put them in an object. 
	 * Relies (as much of this code does) on a regularly formed XML file. Trust has to
	 * start somewhere ;-) 
	 */
	for (var i = 0; i < xmlDoc.getElementsByTagName("name").length; i++)
	    {
		locations[i] = {name:"",type:"",street:"",city:"",state:"",zip:"",latitude:"",longitude:""};
		locations[i].name =      xmlDoc.getElementsByTagName("name")[i].childNodes[0].nodeValue;
		locations[i].type =      xmlDoc.getElementsByTagName("type")[i].childNodes[0].nodeValue;
		locations[i].street =    xmlDoc.getElementsByTagName("street")[i].childNodes[0].nodeValue;
		locations[i].city =      xmlDoc.getElementsByTagName("city")[i].childNodes[0].nodeValue;
		locations[i].state =     xmlDoc.getElementsByTagName("state")[i].childNodes[0].nodeValue;
		locations[i].zip =       xmlDoc.getElementsByTagName("zip")[i].childNodes[0].nodeValue;
		locations[i].latitude =  xmlDoc.getElementsByTagName("latitude")[i].childNodes[0].nodeValue;
		locations[i].longitude = xmlDoc.getElementsByTagName("longitude")[i].childNodes[0].nodeValue;
	    }	
	davespaceGMapFromXML.load();
    }
    catch (e)
    {
	document.getElementById("header").innerHTML = "XML FAULTY: Please Contact WebHost";
	GUnload();
    }
}; // End parseXML()

/* These are used in makeControlPanel() and makeCenterPoints() to sidestep some particularly 
 * irritating apostrophe problems (i.e. Names or types with apostrophes) without
 * using the deprecated "escape()/unescape()" functions. Similar functions would have to be
 * added to implement sorting by other means.
 */
davespaceGMapFromXML.getNameFromNum = function (number){
    return davespaceGMapFromXML.locations[number].name;
};
davespaceGMapFromXML.getTypeFromNum = function (number){
    return davespaceGMapFromXML.locations[number].type;
};

/* Gets all of the different types out of the XML file, and creates
 * controls to show each of these types.  Somewhat Perlish in spirit.
 */
davespaceGMapFromXML.makeControlPanel = function()
{
    // Let's alias the ridiculously long global property names, shall we?
    var locations = davespaceGMapFromXML.locations;
    var xmlDoc = davespaceGMapFromXML.xmlDoc;

    var typenames = {};
    for (var i = 0; i < locations.length; i++){
	typenames[locations[i].type] = i;
    }
    var controls = 
	"<a class='control' onclick=\"davespaceGMapFromXML.getBoundsZoom('ALLALLALL')\">Show All</a><br />";
    for (var name in typenames){
	controls += 
	"<a class='control' onclick=\"davespaceGMapFromXML.getBoundsZoom(davespaceGMapFromXML.getTypeFromNum(" + 
	    typenames[name] + "))\">" + name + "</a><br />";
    }
    document.getElementById("controlPanel").innerHTML = controls;
}; //End makeControlPanel

/* Function that prints list of names at the bottom of the page, 
 * each of which centers the map on the place so named.
 * Also conveniently makes whatever type that place is visible, and hides others.
 */
davespaceGMapFromXML.makeCenterPoints = function ()
{
    // Let's alias the ridiculously long global property names, shall we?
    var locations = davespaceGMapFromXML.locations;
    var markers = davespaceGMapFromXML.markers;
    
    var controls = "Recenter Map on:<br /><a class='control' onclick='davespaceGMapFromXML.getBoundsZoom();"+
                   "'>All</a>, ";
    for (var i = 0; i < locations.length; i++)
    {
	controls += "<a class='control' onclick=" +
	    "'davespaceGMapFromXML.centerOnPlace(" +
	    i + ")'>" +
	    locations[i].name + "</a>";

	// If we're on the last number:
	if (i == (locations.length - 1))
	{
	    controls += ".";
	} 
	// If it's not last, but is the third on its line
	else if (((i + 1) % 3) == 0)
	{
	    controls += ",<br />\n";
	}
	else 
	{
	    controls += ",&nbsp;&nbsp;";
	}
    }
    document.getElementById("centerPoints").innerHTML = controls;
}; // End makeCenterPoints

/* Takes a string.  If the type matches, display markers, if the type does not match, hide them.
 * Makes a special case for ALLALLALL.  Regrettably, this means that ALLALLALL is not a good value
 * for "type" in the XML file.  Caps and repetition are to reduce the likeliness of this already
 * unlikely name collision.
 */
davespaceGMapFromXML.showHide = function(type)
{
    // Aliases for "globals"
    var locations = davespaceGMapFromXML.locations;
    var xmlDoc = davespaceGMapFromXML.xmlDoc;
    var markers = davespaceGMapFromXML.markers;

    for (var i = 0; i < locations.length; i++)
    {
	if (locations[i].type == type || type == "ALLALLALL"){
	    if (markers[i].isHidden())
            {
		markers[i].show();
	    }
	}
	if (locations[i].type != type && type != "ALLALLALL")
	{
	    if (!markers[i].isHidden())
	    {
		markers[i].hide();
	    }    
	}
    }
}; //End showHide

/* Centers the map on a place. */
davespaceGMapFromXML.centerOnPlace = function (number)
{
    // Let's alias the ridiculously long global property names, shall we?
    var locations = davespaceGMapFromXML.locations;
    var markers = davespaceGMapFromXML.markers;

    map.setCenter(davespaceGMapFromXML.markers[number].getLatLng());
    davespaceGMapFromXML.showHide(davespaceGMapFromXML.locations[number].type);
    var content = 
	"<p style='font-size:10pt'>Name: "+ locations[number].name +
                "<br />Type: "+ locations[number].type +
        "<br />Street: "+ locations[number].street +
        "<br />City: " + locations[number].city +
        "<br />State:" + locations[number].state +
        "<br />Zip Code: " + locations[number].zip + "</p>";
    markers[number].openInfoWindowHtml(content);
}; // End centerOnPlace

/* If passed no argumentsigures out what zoom level the map must be at to contain 
 *    all points, and where it should be centered, and then centers it.
 *
 * If passed a string containing the name of a type, does the same, but only taking 
 * points of that type into account. 
 */ 
davespaceGMapFromXML.getBoundsZoom = function (typename)
{
    //Aliases for globals
    var markers = davespaceGMapFromXML.markers;
    var locations = davespaceGMapFromXML.locations;
    
    var bounds = new GLatLngBounds();
    if (arguments.length == 0 || typename == "ALLALLALL"){
	davespaceGMapFromXML.showHide("ALLALLALL");
	for (var i = 0; i < markers.length; i++)
	{ 
	    bounds.extend(markers[i].getLatLng());
	}
    }
    else {
	for (var i = 0; i < markers.length; i++)
	{
	    if (locations[i].type == typename)
	    {
		bounds.extend(markers[i].getLatLng());
	    }
	}
    }
    if (typename)
    {
	davespaceGMapFromXML.showHide(typename);
    }
    map.setZoom(map.getBoundsZoomLevel(bounds) - 1);
    map.setCenter(bounds.getCenter());
    map.closeInfoWindow();
};

/* Does the bulk of the work. Creates the GMap, populates the markers
 * array with markers, and controls the bounds of the map. 
*/
davespaceGMapFromXML.load = function() 
{
    // Aliases for "globals"
    var locations = davespaceGMapFromXML.locations;
    var markers = davespaceGMapFromXML.markers;

    davespaceGMapFromXML.makeControlPanel();
    if (GBrowserIsCompatible()) 
    {
	map = new GMap2(document.getElementById("map"));
	map.setCenter(new GLatLng(locations[0].latitude,locations[0].longitude), 13);
       	for (var i = 0; i < locations.length; i++)
	{
            markers[i] = new GMarker(new GLatLng(locations[i].latitude, locations[i].longitude));
            map.addOverlay(markers[i]);
            var content = 
		"<p style='font-size:10pt'>Name: "+ locations[i].name +
                "<br />Type: "+ locations[i].type +
                "<br />Street: "+ locations[i].street +
                "<br />City: " + locations[i].city +
                "<br />State:" + locations[i].state +
                "<br />Zip Code: " + locations[i].zip + "</p>";
            markers[i].bindInfoWindowHtml(content);
	}
	davespaceGMapFromXML.getBoundsZoom();
	map.enableScrollWheelZoom();
	davespaceGMapFromXML.makeCenterPoints();
	map.addControl(new GLargeMapControl());
	
    }
}; /* End load() */
